# SPDX-License-Identifier: Apache-2.0

import asyncio
import os
import time
import json
import docker
import logging
import unittest

from hfc.fabric.channel.channel import SYSTEM_CHANNEL_NAME

from hfc.util.policies import s2d

from hfc.fabric.client import Client
from test.integration.config import E2E_CONFIG
from test.integration.utils import BaseTestCase

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

CC_PATH = 'github.com/marbles_cc_private'
CC_NAME = 'marbles_cc_private'
CC_VERSION = '1.0'


class E2ePrivateDataTest(BaseTestCase):

    def setUp(self):
        self.gopath_bak = os.environ.get('GOPATH', '')
        gopath = os.path.normpath(os.path.join(os.path.dirname(__file__),
                                               "../fixtures/chaincode"))
        os.environ['GOPATH'] = os.path.abspath(gopath)
        self.channel_tx = \
            E2E_CONFIG['test-network']['channel-artifacts']['channel.tx']
        self.compose_file_path = \
            E2E_CONFIG['test-network']['docker']['compose_file_tls']

        self.config_yaml = \
            E2E_CONFIG['test-network']['channel-artifacts']['config_yaml']
        self.channel_profile = \
            E2E_CONFIG['test-network']['channel-artifacts']['channel_profile']
        self.client = Client('test/fixtures/network.json')
        self.channel_name = "businesschannel"  # default application channel
        self.user = self.client.get_user('org1.example.com', 'Admin')
        self.assertIsNotNone(self.user, 'org1 admin should not be None')

        # Boot up the testing network
        self.shutdown_test_env()
        self.start_test_env()
        time.sleep(1)

    def tearDown(self):
        super(E2ePrivateDataTest, self).tearDown()

    async def channel_create(self):
        """
        Create an channel for further testing.

        :return:
        """
        logger.info(f"E2E: Channel creation start: name={self.channel_name}")

        # By default, self.user is the admin of org1
        response = await self.client.channel_create(
            'orderer.example.com',
            self.channel_name,
            self.user,
            config_yaml=self.config_yaml,
            channel_profile=self.channel_profile)
        self.assertTrue(response)

        logger.info(f"E2E: Channel creation done: name={self.channel_name}")

    async def channel_join(self):
        """
        Join peers of two orgs into an existing channels

        :return:
        """

        logger.info(f"E2E: Channel join start: name={self.channel_name}")

        # channel must already exist when to join
        channel = self.client.get_channel(self.channel_name)
        self.assertIsNotNone(channel)

        orgs = ["org1.example.com", "org2.example.com"]
        for org in orgs:
            org_admin = self.client.get_user(org, 'Admin')
            response = await self.client.channel_join(
                requestor=org_admin,
                channel_name=self.channel_name,
                peers=['peer0.' + org, 'peer1.' + org],
                orderer='orderer.example.com'
            )
            self.assertTrue(response)
            # Verify the ledger exists now in the peer node
            dc = docker.from_env()
            for peer in ['peer0', 'peer1']:
                peer0_container = dc.containers.get(peer + '.' + org)
                code, output = peer0_container.exec_run(
                    'test -f '
                    '/var/hyperledger/production/ledgersData/chains/'
                    f'chains/{self.channel_name}'
                    '/blockfile_000000')
                self.assertEqual(code, 0, "Local ledger not exists")

        logger.info(f"E2E: Channel join done: name={self.channel_name}")

    async def channel_update_anchors(self):

        orgs = ["org1.example.com", "org2.example.com"]

        anchors = [
            'test/fixtures/e2e_cli/channel-artifacts/' + msporg + 'anchors.tx'
            for msporg in ['Org1MSP', 'Org2MSP']
        ]

        for org, anchortx in zip(orgs, anchors):
            org_admin = self.client.get_user(org, "Admin")
            response = await self.client.channel_update(
                "orderer.example.com",
                self.channel_name,
                org_admin,
                config_tx=anchortx
            )

            self.assertTrue(response)

    async def chaincode_install(self):
        """
        Test installing an example chaincode to peer
        """
        logger.info("E2E: Chaincode install start")
        cc = f'/var/hyperledger/production/chaincodes/{CC_NAME}.{CC_VERSION}'

        # uncomment for testing with packaged_cc

        # create packaged chaincode before for having same id
        # code_package = package_chaincode(CC_PATH, CC_TYPE_GOLANG)

        orgs = ["org1.example.com", "org2.example.com"]
        for org in orgs:

            # simulate possible different chaincode archive based on timestamp
            time.sleep(2)

            org_admin = self.client.get_user(org, "Admin")
            responses = await self.client.chaincode_install(
                requestor=org_admin,
                peers=['peer0.' + org, 'peer1.' + org],
                cc_path=CC_PATH,
                cc_name=CC_NAME,
                cc_version=CC_VERSION,
                # packaged_cc=code_package
            )
            self.assertTrue(responses)
            # Verify the cc pack exists now in the peer node
            dc = docker.from_env()
            for peer in ['peer0', 'peer1']:
                peer_container = dc.containers.get(peer + '.' + org)
                code, output = peer_container.exec_run(f'test -f {cc}')
                self.assertEqual(code, 0, "chaincodes pack not exists")

        logger.info("E2E: chaincode install done")

    async def chaincode_instantiate(self):
        """
        Test instantiating an example chaincode to peer
        """
        logger.info("E2E: Chaincode instantiation start")

        org = "org1.example.com"

        policy = s2d().parse("OR('Org1MSP.member', 'Org2MSP.member')")

        collections_config = [
            {
                "name": "collectionMarbles",
                "policy": s2d().parse(
                    "OR('Org1MSP.member','Org2MSP.member')"
                ),
                "requiredPeerCount": 0,
                "maxPeerCount": 1,
                "blockToLive": 1000000,
                "memberOnlyRead": True
            },

            {
                "name": "collectionMarblePrivateDetails",
                "policy": s2d().parse(
                    "OR('Org1MSP.member')"
                ),
                "requiredPeerCount": 0,
                "maxPeerCount": 1,
                "blockToLive": 5,
                "memberOnlyRead": True
            }
        ]

        org_admin = self.client.get_user(org, "Admin")
        response = await self.client.chaincode_instantiate(
            requestor=org_admin,
            channel_name=self.channel_name,
            peers=['peer0.' + org],
            args=None,
            cc_name=CC_NAME,
            cc_version=CC_VERSION,
            cc_endorsement_policy=policy,
            collections_config=collections_config,
            wait_for_event=True,
            wait_for_event_timeout=40
        )
        logger.info(
            "E2E: Chaincode instantiation response {}".format(response))
        policy = {
            'version': 0,
            'rule': {'n_out_of': {
                'n': 1,
                'rules': [
                    {'signed_by': 0},
                    {'signed_by': 1}
                ]}
            },
            'identities': [
                {
                    'principal_classification': 'ROLE',
                    'principal': {
                        'msp_identifier': 'Org1MSP',
                        'role': 'MEMBER'
                    }
                },
                {
                    'principal_classification': 'ROLE',
                    'principal': {
                        'msp_identifier': 'Org2MSP',
                        'role': 'MEMBER'
                    }
                },
            ]
        }

        self.assertEqual(response['name'], CC_NAME)
        self.assertEqual(response['version'], CC_VERSION)
        self.assertEqual(response['policy'], policy)
        logger.info("E2E: chaincode instantiation done")

    async def chaincode_invoke(self):
        """
        Test invoking an example chaincode to peer

        :return:
        """
        logger.info("E2E: Chaincode invoke start")

        orgs = ["org1.example.com"]
        marble = json.dumps({
            "name": "marble1",
            "color": "blue",
            "size": 35,
            "owner": "tom",
            "price": 99
        }).encode()

        for org in orgs:
            org_admin = self.client.get_user(org, "Admin")
            response = await self.client.chaincode_invoke(
                requestor=org_admin,
                channel_name=self.channel_name,
                peers=['peer0.' + org],
                fcn='initMarble',
                args=None,
                cc_name=CC_NAME,
                wait_for_event=True,
                wait_for_event_timeout=40,
                transient_map={"marble": marble}
            )
            self.assertFalse(response)

        # Wait for gossip private data
        time.sleep(5)

        logger.info("E2E: chaincode invoke done")

    async def chaincode_query(self):
        """
        Test invoking an example chaincode to peer

        :return:
        """
        logger.info("E2E: Chaincode query start")

        orgs = ["org1.example.com"]

        args = ["marble1"]

        res = {
            "color": "blue",
            "docType": "marble",
            "name": "marble1",
            "owner": "tom",
            "size": 35
        }

        resp = {
            "docType": "marblePrivateDetails",
            "name": "marble1",
            "price": 99
        }

        for org in ["org1.example.com", "org2.example.com"]:
            org_admin = self.client.get_user(org, "Admin")
            response = await self.client.chaincode_query(
                requestor=org_admin,
                channel_name=self.channel_name,
                peers=['peer0.' + org],
                fcn="readMarble",
                args=args,
                cc_name=CC_NAME
            )
            self.assertEqual(json.loads(response), res)

        orgs = ["org1.example.com"]
        args = ["marble1"]
        for org in orgs:
            org_admin = self.client.get_user(org, "Admin")
            response = await self.client.chaincode_query(
                requestor=org_admin,
                channel_name=self.channel_name,
                peers=['peer0.' + org],
                fcn="readMarblePrivateDetails",
                args=args,
                cc_name=CC_NAME
            )
            self.assertEqual(json.loads(response), resp)

        orgs = ["org2.example.com"]
        args = ["marble1"]
        error = "does not have read access permission"
        for org in orgs:
            org_admin = self.client.get_user(org, "Admin")
            with self.assertRaises(Exception) as context:
                response = await self.client.chaincode_query(
                    requestor=org_admin,
                    channel_name=self.channel_name,
                    peers=['peer0.' + org],
                    fcn="readMarblePrivateDetails",
                    args=args,
                    cc_name=CC_NAME
                )
                self.assertTrue(error in str(context.exception))

        logger.info("E2E: chaincode query done")

    async def query_installed_chaincodes(self):
        """
        Test query installed chaincodes on peer

        :return:
        """
        logger.info("E2E: Query installed chaincode start")

        orgs = ["org1.example.com", "org2.example.com"]
        for org in orgs:
            org_admin = self.client.get_user(org, "Admin")
            responses = await self.client.query_installed_chaincodes(
                requestor=org_admin,
                peers=['peer0.' + org, 'peer1.' + org],
            )
            self.assertEqual(
                responses[0].chaincodes[0].name, CC_NAME, "Query failed")
            self.assertEqual(
                responses[0].chaincodes[0].version, CC_VERSION, "Query failed")
            self.assertEqual(
                responses[0].chaincodes[0].path, CC_PATH, "Query failed")

        logger.info("E2E: Query installed chaincode done")

    async def query_instantiated_chaincodes(self):
        """
        Test query instantiated chaincodes on peer

        :return:
        """
        logger.info("E2E: Query instantiated chaincode start")

        orgs = ["org1.example.com", "org2.example.com"]
        for org in orgs:
            org_admin = self.client.get_user(org, "Admin")

            responses = await self.client.query_instantiated_chaincodes(
                requestor=org_admin,
                channel_name=self.channel_name,
                peers=['peer0.' + org, 'peer1.' + org],
            )
            self.assertTrue(len(responses) >= 1)
            self.assertEqual(
                responses[0].chaincodes[0].name, CC_NAME, "Query failed")
            self.assertEqual(
                responses[0].chaincodes[0].version, CC_VERSION, "Query failed")
            self.assertEqual(
                responses[0].chaincodes[0].path, CC_PATH, "Query failed")

        logger.info("E2E: Query installed chaincode done")

    async def get_channel_config(self):
        """
        Test get channel config on peer

        :return:
        """
        logger.info(f"E2E: Get channel {self.channel_name} config start")

        orgs = ["org1.example.com"]
        for org in orgs:
            org_admin = self.client.get_user(org, "Admin")
            responses = await self.client.get_channel_config(
                requestor=org_admin,
                channel_name=self.channel_name,
                peers=['peer0.' + org, 'peer1.' + org]
            )
            self.assertEqual(responses[0].config.sequence,
                             1, "Get Config Failed")

        logger.info("E2E: Query installed chaincode done")

    async def get_channel_config_with_orderer(self,
                                              chname=SYSTEM_CHANNEL_NAME):
        """
        Test get channel config on orderer
         :return:
        """
        logger.info(f"E2E: Get channel {chname} config start")

        orgs = ["orderer.example.com"]
        for org in orgs:
            org_admin = self.client.get_user(org, "Admin")
            response = await self.client.get_channel_config_with_orderer(
                orderer='orderer.example.com',
                requestor=org_admin,
                channel_name=chname,
            )
            self.assertEqual(response['config']['sequence'],
                             '0', "Get Config Failed")

        logger.info(f"E2E: Get channel {chname} config done")

    def test_in_sequence(self):

        loop = asyncio.get_event_loop()

        logger.info("\n\nE2E testing started...")

        self.client.new_channel(SYSTEM_CHANNEL_NAME)

        loop.run_until_complete(self.get_channel_config_with_orderer())

        loop.run_until_complete(self.channel_create())

        loop.run_until_complete(self.channel_join())

        loop.run_until_complete(self.channel_update_anchors())

        loop.run_until_complete(self.get_channel_config())

        loop.run_until_complete(self.chaincode_install())

        loop.run_until_complete(self.query_installed_chaincodes())

        loop.run_until_complete(self.chaincode_instantiate())

        loop.run_until_complete(self.query_instantiated_chaincodes())

        loop.run_until_complete(self.chaincode_invoke())

        loop.run_until_complete(self.chaincode_query())

        logger.info("E2E private data all test cases done\n\n")


if __name__ == "__main__":
    unittest.main()



# Copyright IBM Corp. 2016 All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import base64
import sys
import logging
import random
import os
import tarfile
import io
import time

from google.protobuf.message import DecodeError
from google.protobuf.timestamp_pb2 import Timestamp
from hfc.protos.common import common_pb2, configtx_pb2
from hfc.protos.msp import identities_pb2
from hfc.protos.peer import proposal_pb2, chaincode_pb2
from hfc.protos.utils import create_tx_payload

CC_INSTALL = "install"
CC_TYPE_GOLANG = "GOLANG"

_logger = logging.getLogger(__name__)


def proto_str(x):
    return proto_b(x).decode("utf-8")


proto_b = \
    sys.version_info[0] < 3 and (lambda x: x) or (
        lambda x: x.encode('latin1'))


def create_serialized_identity(user):
    """Create serialized identity from user.

    :param user: The user object that should be serialized.
    :type user: user object
    :return: Protobuf SerializedIdentity of
            the given user object.
    :rtype: serialized_identity
    """
    serialized_identity = identities_pb2.SerializedIdentity()
    serialized_identity.mspid = user.msp_id
    serialized_identity.id_bytes = user.enrollment.cert
    return serialized_identity.SerializeToString()


def build_header(creator, channel_header, nonce):
    """This function will build the common header.

    :param creator: Serialized identity of the creator.
    :type creator: protobuf SerializedIdentity
    :param channel_header: ChannelHeader
    :type channel_header: protobuf ChannelHeader
    :param nonce: Nonce that has been used for the tx_id.
    :type nonce: str
    :return: Returns created protobuf common header.
    :rtype: header
    """
    signature_header = common_pb2.SignatureHeader()
    signature_header.creator = creator
    signature_header.nonce = nonce

    header = common_pb2.Header()
    header.signature_header = signature_header.SerializeToString()
    header.channel_header = channel_header.SerializeToString()

    return header


def build_channel_header(type, tx_id, channel_id,
                         timestamp, epoch=0, extension=None,
                         tls_cert_hash=None):
    """Build channel header.

    :param type: type
    :type type: common_pb2.HeaderType
    :param tx_id: transaction id
    :type tx_id: str
    :param channel_id: channel id
    :type channel_id: str
    :param timestamp: timestamp
    :type timestamp: grpc.timestamp
    :param epoch: epoch (Default value = 0)
    :type epoch: int
    :param extension: extension (Default value = None)
    :param tls_cert_hash:  (Default value = None)
    :return: common_proto.Header instance
    """
    channel_header = common_pb2.ChannelHeader()
    channel_header.type = type
    channel_header.version = 1
    channel_header.channel_id = proto_str(channel_id)
    channel_header.tx_id = proto_str(tx_id)
    channel_header.epoch = epoch
    channel_header.timestamp.CopyFrom(timestamp)

    if tls_cert_hash:
        channel_header.tls_cert_hash = tls_cert_hash

    if extension:
        channel_header.extension = extension
    return channel_header


def string_to_signature(string_signatures):
    """Check if signatures are already in protobuf format.

    :param string_signatures: An list of protobuf ConfigSignatures either
            represented as or serialized as byte strings.
    :type string_signatures: list
    :returns: List of protobuf ConfigSignatures.
    :rtype: list
    """
    signatures = []

    for signature in string_signatures:
        if signature and hasattr(signature, 'header') \
                and hasattr(signature, 'signature'):
            _logger.debug('_string_to_signature - signature is protobuf')
            config_signature = signature

        else:
            _logger.debug('_string_to_signature - signature is string')

            config_signature = configtx_pb2.ConfigSignature()
            config_signature.ParseFromString(signature)

        signatures.append(config_signature)

    return signatures


def current_timestamp():
    """Get current timestamp.

    :return: Current timestamp.
    """
    timestamp = Timestamp()
    timestamp.GetCurrentTime()
    return timestamp


def extract_channel_config(configtx_proto_envelope):
    """Extracts the protobuf 'ConfigUpdate' object out ouf the 'ConfigEnvelope'.

    :param configtx_proto_envelope: The encoded bytes of the
            ConfigEnvelope protofbuf.
    :type configtx_proto_envelope: common_pb2.Envelope
    :return: (config_update) The encoded bytes of the ConfigUpdate protobuf, ready to be signed
    :rtype: configtx_pb2.ConfigUpadeEnvelope.config_update
    :raises ValueError: If there is an error in protobuf_decode due to a wrong or
            not valid profobuf file a ValueError is raised.

    """
    _logger.debug('extract_channel_config - start')

    try:
        envelope = common_pb2.Envelope()
        envelope.ParseFromString(configtx_proto_envelope)

        payload = common_pb2.Payload()
        payload.ParseFromString(envelope.payload)

        configtx = configtx_pb2.ConfigUpdateEnvelope()
        configtx.ParseFromString(payload.data)

    except DecodeError as e:
        _logger.error('extract_channel_config - an error occurred decoding'
                      ' the configtx_proto_envelope: {}'.format(e))
        raise ValueError('The given configtx_proto_envelope was not valid: {}'
                         .format(e))

    return configtx.config_update


def build_cc_proposal(cci_spec, header, transient_map):
    """Create an chaincode transaction proposal

    :param transient_map: transient data map
    :param cci_spec: The spec
    :param header: header of the proposal
    :return: The created proposal
    """
    cc_payload = proposal_pb2.ChaincodeProposalPayload()
    cc_payload.input = cci_spec.SerializeToString()
    if transient_map:
        for name, bytes_value in transient_map.items():
            cc_payload.TransientMap[name] = bytes_value

    proposal = proposal_pb2.Proposal()
    proposal.header = header.SerializeToString()
    proposal.payload = cc_payload.SerializeToString()

    return proposal


def sign_proposal(tx_context, proposal):
    """Sign a proposal

    :param tx_context: transaction context
    :param proposal: proposal to sign on
    :return: Signed proposal
    """
    proposal_bytes = proposal.SerializeToString()
    sig = tx_context.sign(proposal_bytes)

    signed_proposal = proposal_pb2.SignedProposal()
    signed_proposal.signature = sig
    signed_proposal.proposal_bytes = proposal_bytes

    return signed_proposal


def send_transaction_proposal(proposal, tx_context, peers):
    """Send transaction proposal

    :param header: header
    :param tx_context: transaction context
    :param proposal: transaction proposal
    :param peers: peers
    :return: a list containing all the proposal response
    """
    signed_proposal = sign_proposal(tx_context, proposal)

    send_executions = [peer.send_proposal(signed_proposal)
                       for peer in peers]

    return send_executions


def send_transaction(orderers, tran_req, tx_context):
    """Send a transaction to the chain's orderer service (one or more
    orderer endpoints) for consensus and committing to the ledger.

    This call is asynchronous and the successful transaction commit is
    notified via a BLOCK or CHAINCODE event. This method must provide a
    mechanism for applications to attach event listeners to handle
    'transaction submitted', 'transaction complete' and 'error' events.

    :param tx_context: transaction context
    :param orderers: orderers
    :param tran_req: The transaction object
    :type tran_req: TransactionRequest
    :return: (EventEmitter) an handle to allow the application to attach event handlers on 'submitted',
            'complete', and 'error'.
    :rtype: EventEmitter
    """
    if not tran_req:
        _logger.warning("Missing input request object on the transaction "
                        "request")
        raise ValueError(
            "Missing input request object on the transaction request"
        )

    if not tran_req.responses or len(tran_req.responses) < 1:
        _logger.warning("Missing 'proposalResponses' parameter in transaction "
                        "request")
        raise ValueError(
            "Missing 'proposalResponses' parameter in transaction request"
        )

    if not tran_req.proposal:
        _logger.warning("Missing 'proposalResponses' parameter in transaction "
                        "request")
        raise ValueError(
            "Missing 'proposalResponses' parameter in transaction request"
        )

    if len(orderers) < 1:
        _logger.warning("Missing orderer objects on this chain")
        raise ValueError(
            "Missing orderer objects on this chain"
        )

    endorsements = map(lambda res: res.endorsement, tran_req.responses)

    tran_payload_bytes = create_tx_payload(endorsements, tran_req)
    envelope = sign_tran_payload(tx_context, tran_payload_bytes)

    if sys.version_info < (3, 0):
        orderer = random.choice(orderers.values())
    else:
        orderer = random.choice(list(orderers.values()))
    return orderer.broadcast(envelope)


def sign_tran_payload(tx_context, tran_payload_bytes):
    """Sign a transaction payload

    :param signing_identity: id to sign with
    :param tran_payload: transaction payload to sign on
    :param tx_context:
    :param tran_payload_bytes:
    :return: Envelope
    """
    sig = tx_context.sign(tran_payload_bytes)

    envelope = common_pb2.Envelope()
    envelope.signature = sig
    envelope.payload = tran_payload_bytes

    return envelope


def build_tx_req(ProposalResponses):
    """Check the endorsements from peers

    :param reponses: ProposalResponse from endorsers
    :param ProposalResponses:
    :return: an instance of TXRequest
    """

    class TXRequest(object):

        def __init__(self, responses, proposal, header):
            self._responses = responses
            self._proposal = proposal
            self._header = header

        @property
        def responses(self):
            return self._responses

        @property
        def proposal(self):
            return self._proposal

        @property
        def header(self):
            return self._header

    responses, proposal, header = ProposalResponses
    return TXRequest(responses, proposal, header)


def send_install_proposal(tx_context, peers):
    """Send install chaincode proposal

    :param tx_context: transaction context
    :param peers: peers to install chaincode
    :return: a set of proposal response
    """

    if not tx_context:
        raise ValueError("InstallProposalRequest is empty.")

    if not peers:
        raise ValueError("Please specify the peer.")

    cc_deployment_spec = chaincode_pb2.ChaincodeDeploymentSpec()
    cc_deployment_spec.chaincode_spec.type = \
        chaincode_pb2.ChaincodeSpec.Type.Value(
            proto_str(tx_context.tx_prop_req.cc_type))
    cc_deployment_spec.chaincode_spec.chaincode_id.name = \
        proto_str(tx_context.tx_prop_req.cc_name)
    cc_deployment_spec.chaincode_spec.chaincode_id.path = \
        proto_str(tx_context.tx_prop_req.cc_path)
    cc_deployment_spec.chaincode_spec.chaincode_id.version = \
        proto_str(tx_context.tx_prop_req.cc_version)

    if not tx_context.tx_prop_req.packaged_cc:
        cc_deployment_spec.code_package = \
            package_chaincode(
                tx_context.tx_prop_req.cc_path,
                tx_context.tx_prop_req.cc_type)
    else:
        cc_deployment_spec.code_package = \
            tx_context.tx_prop_req.packaged_cc

    channel_header_extension = proposal_pb2.ChaincodeHeaderExtension()
    channel_header_extension.chaincode_id.name = \
        proto_str("lscc")
    channel_header = build_channel_header(
        common_pb2.ENDORSER_TRANSACTION,
        tx_context.tx_id,
        '',
        current_timestamp(),
        tx_context.epoch,
        channel_header_extension.SerializeToString()
    )

    header = build_header(tx_context.identity,
                          channel_header,
                          tx_context.nonce)

    cci_spec = chaincode_pb2.ChaincodeInvocationSpec()
    cci_spec.chaincode_spec.type = \
        chaincode_pb2.ChaincodeSpec.Type.Value(CC_TYPE_GOLANG)
    cci_spec.chaincode_spec.chaincode_id.name = proto_str("lscc")
    cci_spec.chaincode_spec.input.args.extend(
        [proto_b(CC_INSTALL), cc_deployment_spec.SerializeToString()])
    proposal = build_cc_proposal(
        cci_spec, header,
        tx_context.tx_prop_req.transient_map)
    signed_proposal = sign_proposal(tx_context, proposal)

    responses = [peer.send_proposal(signed_proposal) for peer in peers]
    return responses, proposal, header


# https://jira.hyperledger.org/browse/FAB-7065
# ?page=com.atlassian.jira.plugin.system.issuetabpanels%3Acomment
# -tabpanel&showAll=true
def zeroTarInfo(tarinfo):

    tarinfo.uid = tarinfo.gid = 500
    tarinfo.mode = 100644
    tarinfo.mtime = 0
    tarinfo.pax_headers = {
        'atime': 0,
        'ctime': 0,
    }
    return tarinfo


# http://www.onicos.com/staff/iz/formats/gzip.html
# https://github.com/python/cpython/blob/master/Lib/tarfile.py#L420
class zeroTimeContextManager(object):
    def __enter__(self):
        self.real_time = time.time
        time.time = lambda: 0

    def __exit__(self, type, value, traceback):
        time.time = self.real_time


def package_chaincode(cc_path, cc_type=CC_TYPE_GOLANG):
    """Package all chaincode env into a tar.gz file

    :param cc_path: path to the chaincode
    :param cc_type: chaincode type (Default value = CC_TYPE_GOLANG)
    :return: The chaincode pkg path or None
    """
    _logger.debug('Packaging chaincode path={}, chaincode type={}'.format(
        cc_path, cc_type))

    if cc_type == CC_TYPE_GOLANG:
        go_path = os.environ['GOPATH']
        if not cc_path:
            raise ValueError("Missing chaincode path parameter "
                             "in install proposal request")

        if not go_path:
            raise ValueError("No GOPATH env variable is found")

        proj_path = go_path + '/src/' + cc_path
        _logger.debug('Project path={}'.format(proj_path))

        if not os.listdir(proj_path):
            raise ValueError("No chaincode file found!")

        tar_stream = io.BytesIO()
        with zeroTimeContextManager():
            dist = tarfile.open(fileobj=tar_stream,
                                mode='w|gz', format=tarfile.GNU_FORMAT)
            for dir_path, _, file_names in os.walk(proj_path):
                for filename in file_names:
                    file_path = os.path.join(dir_path, filename)

                    with open(file_path, mode='rb') as f:
                        arcname = os.path.relpath(file_path, go_path)
                        tarinfo = dist.gettarinfo(file_path, arcname)
                        tarinfo = zeroTarInfo(tarinfo)
                        dist.addfile(tarinfo, f)

            dist.close()
            tar_stream.seek(0)
            code_content = tar_stream.read()

        if code_content:
            return code_content
        else:
            raise ValueError('No chaincode found')

    else:
        raise ValueError('Currently only support install GOLANG chaincode')


def pem_to_der(pem):
    arr = pem.split(b'\n')
    der = b''.join(arr[1:-2])
    return base64.b64decode(der)


# create artificial envelope stream
async def stream_envelope(envelope):
    yield envelope



# Copyright IBM Corp. 2016 All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import hashlib
import hmac
import sys
from abc import ABCMeta, abstractmethod

import six
from Cryptodome import Random
from Cryptodome.Cipher import AES
from cryptography import x509
from cryptography.hazmat.backends \
    import default_backend
from cryptography.hazmat.primitives import constant_time
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.asymmetric import ec
from cryptography.hazmat.primitives.asymmetric.ec \
    import EllipticCurvePublicKey
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric.utils \
    import decode_dss_signature, encode_dss_signature
from cryptography.exceptions import InvalidSignature
from hkdf import Hkdf

if sys.version_info < (3, 6):
    import sha3  # noqa: F401

DEFAULT_NONCE_SIZE = 24

CURVE_P_256_Size = 256
CURVE_P_384_Size = 384

SHA2 = "SHA2"
SHA3 = "SHA3"

AES_KEY_LENGTH = 32
HMAC_KEY_LENGTH = 32
IV_LENGTH = 16


@six.add_metaclass(ABCMeta)
class Key(object):
    """An abstract base class for Key.

    Key represents a base cryptographic key. It can be symmetric or asymmetric.
    In asymmetric case, the private key can retrieve public key with the
    corresponding method.

    A key can be referenced via the Subject Key Identifier (SKI) with DER or
    PEM encoding.
    """

    @abstractmethod
    def is_symmetric(self):
        """ Return if this key is with symmetric crypt, i.e. whether it's a
        symmetric key.

        :return: True or False
        """

    @abstractmethod
    def get_SKI(self):
        """ Return the SKI string

        :return: string represent the SKI
        """


@six.add_metaclass(ABCMeta)
class AsymmetricKey(Key):
    """An asymmetric key.

    Can be a public key or private key, the private key can retrieve public
    key with the corresponding method.
    """

    @abstractmethod
    def is_private(self):
        """ Return if this key is private key

        :return: True or False
        """

    @abstractmethod
    def get_public_key(self):
        """Get the corresponding public key for this private key.

        If this key is already a public one, then return itself.

        :return: Public key
        """


@six.add_metaclass(ABCMeta)
class Crypto(object):
    """An abstract base class for crypto."""

    @abstractmethod
    def generate_private_key(self):
        """Generate asymmetric key pair.

        :return: An private key object which include public key object.
        """

    @abstractmethod
    def encrypt(self, public_key, message):
        """Encrypt the message by encryption public key.

        :param public_key: Encryption public key
        :param message: message need encrypt
        :return: An object including secure context
        """

    @abstractmethod
    def decrypt(self, private_key, cipher_text):
        """Decrypt the cipher text by encryption private key.

        :param private_key: Encryption private key
        :param cipher_text: Cipher text received
        :return: An object including secure context
        """

    @abstractmethod
    def sign(self, private_key, message):
        """Sign the origin message by signing private key.

        :param private_key: Signing private key
        :param message: Origin message
        :return: An object including secure context
        """

    @abstractmethod
    def verify(self, public_key, message, signature):
        """Verify the signature by signing public key.

        :param public_key: Signing public key
        :param message: Origin message
        :param signature: Signature of message
        :return: A boolean True as valid
        """

    @staticmethod
    def generate_nonce(size):
        """Generate a secure random for cryptographic use.

        :param size: Number of bytes for the nonce
        :return: Generated random bytes
        """
        return Random.get_random_bytes(size)


def generate_nonce(size):
    # TODO still has old dependencies but has to be deleted
    """ Generate a secure random for cryptographic use.

    :param size: Number of bytes for the nonce
    :return: Generated random bytes
    """
    return Random.get_random_bytes(size)


class Ecies(Crypto):
    """A crypto implementation based on ECDSA and SHA."""

    def __init__(self, security_level=CURVE_P_256_Size, hash_algorithm=SHA2):
        """ Init curve and hash function.

        :param security_level: security level
        :param hash_algorithm: hash function
        :return: an instance of Ecies
        """
        if security_level == CURVE_P_256_Size:
            # order = openssl.backend._lib.BN_new()
            # curve = openssl.backend._lib.EC_GROUP_new_by_curve_name(
            #     openssl.backend._lib.NID_X9_62_prime256v1)
            # openssl.backend._lib.EC_GROUP_get_order(
            #     curve, order, openssl.backend._ffi.NULL)
            self.order = int("115792089210356248762697446949407573529"
                             "996955224135760342422259061068512044369")
            self.half_order = self.order >> 1
            self.curve = ec.SECP256R1
            self.sign_hash_algorithm = hashes.SHA256()
        else:
            # order = openssl.backend._lib.BN_new()
            # curve = openssl.backend._lib.EC_GROUP_new_by_curve_name(
            #     openssl.backend._lib.NID_secp384r1)
            # openssl.backend._lib.EC_GROUP_get_order(
            #     curve, order, openssl.backend._ffi.NULL)
            self.order = int("39402006196394479212279040100"
                             "14361380507973927046544666794"
                             "69052796276593991132635693989"
                             "56308152294913554433653942643")
            self.half_order = self.order >> 1
            self.curve = ec.SECP384R1
            self.sign_hash_algorithm = hashes.SHA384()

        if hash_algorithm == SHA2:
            self._hash = hashlib.sha256
        elif hash_algorithm == SHA3 and security_level == CURVE_P_256_Size:
            self._hash = hashlib.sha3_256
        else:
            self._hash = hashlib.sha3_384

    @property
    def hash(self):
        """Get hash function

        :return: hash function
        """
        return self._hash

    def sign(self, private_key, message):
        """ECDSA sign message.

        :param private_key: private key
        :param message: message to sign
        :return: signature
        """
        signer = private_key.sign(message, ec.ECDSA(self.sign_hash_algorithm))
        return self._prevent_malleability(signer)

    def verify(self, public_key, message, signature):
        """ECDSA verify signature.

        :param public_key: Signing public key
        :param message: Origin message
        :param signature: Signature of message
        :return: verify result boolean, True means valid
        """
        if not (self._check_malleability(signature)):
            return False
        try:
            public_key.verify(signature, message,
                              ec.ECDSA(self.sign_hash_algorithm))
        except InvalidSignature:
            return False
        except Exception as e:
            raise e
        return True

    def _prevent_malleability(self, sig):
        r, s = decode_dss_signature(sig)
        if s > self.half_order:
            s = self.order - s
        return encode_dss_signature(r, s)

    def _check_malleability(self, sig):
        r, s = decode_dss_signature(sig)
        if s > self.half_order:
            return False
        return True

    def generate_private_key(self):
        """ECDSA key pair generation by current curve.

        :return: A private key object which include public key object.
        """
        return ec.generate_private_key(self.curve, default_backend())

    def decrypt(self, private_key, cipher_text):
        """ECIES decrypt cipher text.

        First restore the ephemeral public key from bytes(97 bytes for 384,
         65 bytes for 256).
        Then derived a shared key based ecdh, using the key based hkdf to
        generate aes key and hmac key,
        using hmac-sha3 to verify the hmac bytes.
        Last using aes-256-cfb to decrypt the bytes.

        :param private_key: private key
        :param cipher_text: cipher text
        :return: plain text
        """
        key_len = private_key.curve.key_size
        if key_len != self.curve.key_size:
            raise ValueError(
                "Invalid key. Input security level {} does not "
                "match the current security level {}".format(
                    key_len,
                    self.curve.key_size))

        d_len = key_len >> 3
        rb_len = ((key_len + 7) // 8) * 2 + 1
        ct_len = len(cipher_text)
        if ct_len <= rb_len + d_len:
            raise ValueError(
                "Illegal cipherText length: cipher text length {} "
                "must be > rb length plus d_len {}".format(ct_len,
                                                           rb_len + d_len)
            )

        rb = cipher_text[:rb_len]
        em = cipher_text[rb_len:ct_len - d_len]
        d = cipher_text[ct_len - d_len:ct_len]

        ephemeral_public_key = EllipticCurvePublicKey \
            .from_encoded_point(self.curve(), rb)
        z = private_key.exchange(ec.ECDH(), ephemeral_public_key)
        hkdf_output = Hkdf(salt=None, input_key_material=z, hash=self._hash) \
            .expand(length=AES_KEY_LENGTH + HMAC_KEY_LENGTH)
        aes_key = hkdf_output[:AES_KEY_LENGTH]
        hmac_key = hkdf_output[AES_KEY_LENGTH:AES_KEY_LENGTH +
                               HMAC_KEY_LENGTH]

        mac = hmac.new(hmac_key, em, self._hash)
        recovered_d = mac.digest()
        if not constant_time.bytes_eq(recovered_d, d):
            raise ValueError("Hmac verify failed.")

        iv = em[:IV_LENGTH]
        aes_cipher = AES.new(key=aes_key, mode=AES.MODE_CFB, iv=iv)
        return aes_cipher.decrypt(em[IV_LENGTH:len(em)])

    def encrypt(self, public_key, plain_text):
        """ECIES encrypt plain text.

        First create a ephemeral ecdsa key pair, then serialize the public
        key for part of result.
        Then derived a shared key based ecdh, using the key based hkdf to
        generate aes key and hmac key,
        using aes-256-cfb to generate the part of result.
        Last using hmac-sha3 and the part of previous step to generate
        last part
        of result.

        :param public_key: public key
        :param plain_text: plain text
        :return: cipher text
        """
        ephemeral_private_key = self.generate_private_key()
        rb = ephemeral_private_key.public_key().public_bytes(
            serialization.Encoding.X962,
            serialization.PublicFormat.UncompressedPoint
        )

        z = ephemeral_private_key.exchange(ec.ECDH(), public_key)
        hkdf_output = Hkdf(salt=None, input_key_material=z, hash=self._hash) \
            .expand(length=AES_KEY_LENGTH + HMAC_KEY_LENGTH)
        aes_key = hkdf_output[:AES_KEY_LENGTH]
        hmac_key = hkdf_output[AES_KEY_LENGTH:AES_KEY_LENGTH +
                               HMAC_KEY_LENGTH]

        aes_cipher = AES.new(aes_key, AES.MODE_CFB)
        em = aes_cipher.iv + aes_cipher.encrypt(plain_text)
        mac = hmac.new(hmac_key, em, self._hash)
        d = mac.digest()

        return rb + em + d

    def generate_csr(self, private_key, subject_name, extensions=None):
        """Generate certificate signing request.

        :param private_key: Private key
        :param subject_name: Subject name
        :type subject_name: x509.Name
        :param extensions:  (Default value = None)
        return: x509.CertificateSigningRequest
        """
        builder = x509.CertificateSigningRequestBuilder(
            subject_name, [] if extensions is None else extensions)

        return builder.sign(
            private_key, self.sign_hash_algorithm, default_backend())


def ecies(security_level=CURVE_P_256_Size, hash_algorithm=SHA2):
    """Factory method for creating a Ecies instance.

    :param security_level: Security level (Default value = CURVE_P_256_Size)
    :param hash_algorithm: Hash algorithm
    :return: A Ecies instance (Default value = SHA2)
    """
    return Ecies(security_level, hash_algorithm)



# Copyright IBM Corp. 2017 All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import logging
from _sha256 import sha256

from hfc.protos.common import common_pb2
from hfc.protos.orderer import ab_pb2_grpc
from hfc.protos.utils import create_seek_info, create_seek_payload, \
    create_envelope
from hfc.util.channel import create_grpc_channel

from hfc.util.utils import current_timestamp, \
    build_header, build_channel_header, stream_envelope, pem_to_der

DEFAULT_ORDERER_ENDPOINT = 'localhost:7050'

_logger = logging.getLogger(__name__ + ".orderer")


class Orderer(object):
    """A orderer node in the network. It has a specific grpc channel address.
    :param object:
    :type object:
    """

    def __init__(self, name='orderer', endpoint=DEFAULT_ORDERER_ENDPOINT,
                 tls_ca_cert_file=None, client_key_file=None,
                 client_cert_file=None, opts=None):
        """Creates an orderer object.

        :param name: defaults to 'orderer'
        :type name: str
        :param endpoint: The grpc endpoint of the orderer, defaults to DEFAULT_ORDERER_ENDPOINT
        :type endpoint: str
        :param tls_ca_cert_file: The tls certificate for the given orderer as bytes, defaults to None
        :type tls_ca_cert_file: str
        :param client_key_file: file path for Private key used for TLS when making client connections,
                                defaults to None
        :type client_key_file: str
        :param client_cert_file: file path for X.509 certificate used for TLS when making client connections,
                                 defaults to None
        :type client_cert_file: str
        :param opts: Additional grpc config options as tuple e.g. ((key, val),), defaults to None
        :type opts: tuple
        """

        self._name = name
        self._endpoint = endpoint
        if opts:
            self._grpc_options = {key: value for (key, value) in opts}
        else:
            self._grpc_options = dict()
        self._ssl_target_name = None
        self._tls_ca_certs_path = tls_ca_cert_file
        self._client_key_path = client_key_file
        self._client_cert_path = client_cert_file
        self._channel = create_grpc_channel(self._endpoint, tls_ca_cert_file,
                                            client_key_file,
                                            client_cert_file, opts)
        self._orderer_client = ab_pb2_grpc.AtomicBroadcastStub(self._channel)

    def init_with_bundle(self, info):
        """Init the peer with given info dict

        :param info: Dict including all info, e.g., endpoint, grpc option
        :return: True/False
        :rtype: Boolean
        """

        try:
            self._endpoint = info['url']
            self._grpc_options = info['grpcOptions']
            self._tls_ca_certs_path = info['tlsCACerts']['path']
            if 'clientKey' in info:
                self._client_key_path = info['clientKey']['path']
            if 'clientCert' in info:
                self._client_cert_path = info['clientCert']['path']
            self._ssl_target_name = self._grpc_options[
                'grpc.ssl_target_name_override']
            self._channel = create_grpc_channel(
                self._endpoint,
                self._tls_ca_certs_path,
                self._client_key_path,
                self._client_cert_path,
                opts=[(opt, value) for opt, value in
                      self._grpc_options.items()])
            self._orderer_client = ab_pb2_grpc.AtomicBroadcastStub(
                self._channel)
        except KeyError as e:
            print(e)
            return False
        return True

    def get_genesis_block(self, tx_context, channel_name):
        """get the genesis block of the channel

        :return: the genesis block in success or None in fail
        :rtype: Block/None
        """
        _logger.info("get genesis block - start")

        seek_info = create_seek_info(0, 0)

        kwargs = {}
        if self._client_cert_path:
            with open(self._client_cert_path, 'rb') as f:
                b64der = pem_to_der(f.read())
                kwargs['tls_cert_hash'] = sha256(b64der).digest()

        seek_info_header = build_channel_header(
            common_pb2.HeaderType.Value('DELIVER_SEEK_INFO'),
            tx_context.tx_id,
            channel_name,
            current_timestamp(),
            tx_context.epoch,
            **kwargs
        )

        seek_header = build_header(
            tx_context.identity,
            seek_info_header,
            tx_context.nonce)

        seek_payload_bytes = create_seek_payload(seek_header, seek_info)
        sig = tx_context.sign(seek_payload_bytes)
        envelope = create_envelope(sig, seek_payload_bytes)

        # this is a stream response
        return self.delivery(envelope)

    def broadcast(self, envelope):
        """Send an broadcast envelope to orderer.

        :param envelope: The message envelope
        :return: orderer_response or exception
        """

        _logger.debug("Send envelope={}".format(envelope))

        # this is a stream response
        return self._orderer_client.Broadcast(stream_envelope(envelope))

    def delivery(self, envelope, scheduler=None):
        """Send an delivery envelop to orderer.

        :param envelope: The message envelope
        :param scheduler: defaults to None
        :return: orderer_response or exception
        """

        _logger.debug("Send envelope={}".format(envelope))

        # this is a stream response
        return self._orderer_client.Deliver(stream_envelope(envelope))

    def get_attrs(self):
        return ",".join("{}={}"
                        .format(k, getattr(self, k))
                        for k in self.__dict__.keys())

    def __str__(self):
        return "[{}:{}]".format(self.__class__.__name__, self.get_attrs())

    @property
    def endpoint(self):
        """Return the endpoint of the orderer.

        :return: endpoint
        """
        return self._endpoint

    @property
    def name(self):
        """Return the name of the orderer.

        :return: name of orderer
        :rtype: str
        """

        return self._name

    def _handle_response_stream(self, responses):
        """Handle response stream.

        :param responses: responses
        :return: response
        :rtype: tuple
        """

        for response in responses:
            return response, self

    def set_tls_client_cert_and_key(self, client_key_file=None,
                                    client_cert_file=None):
        """Set tls client's cert and key for mutual tls

        :param client_key_file: file path for Private key used for TLS when making client connections,
                                defaults to None
        :type client_key_file: str
        :param client_cert_file: file path for X.509 certificate used for TLS when making client connections,
                                 defaults to None
        :type client_cert_file: str
        :return: set success value
        :rtype: Boolean
        """

        try:
            self._client_key_path = client_key_file
            self._client_cert_path = client_cert_file
            self._channel = create_grpc_channel(
                self._endpoint,
                self._tls_ca_certs_path,
                self._client_key_path,
                self._client_cert_path,
                opts=[(opt, value) for opt, value in
                      self._grpc_options.items()])
            self._orderer_client = ab_pb2_grpc.AtomicBroadcastStub(
                self._channel)
        except Exception:
            return False
        return True



import unittest

from hfc.util.policies import s2d, d2s


class PoliciesTest(unittest.TestCase):

    def setUp(self):
        self.outof_or = "OutOf(1, 'Org1.member', 'Org2.member')"
        # is equivalent to
        self.or_ = "OR('Org1.member', 'Org2.member')"
        self.or_d = {'identities': [
            {'role': {'mspId': 'Org1', 'name': 'member'}},
            {'role': {'mspId': 'Org2', 'name': 'member'}}
        ],
            'policy': {'1-of': [{'signed-by': 0}, {'signed-by': 1}]}}

        self.outof_and = "OutOf(2, 'Org1.member', 'Org2.member')"
        # is equivalent to
        self.and_ = "AND('Org1.member', 'Org2.member')"
        self.and_d = {'identities': [
            {'role': {'mspId': 'Org1', 'name': 'member'}},
            {'role': {'mspId': 'Org2', 'name': 'member'}}
        ],
            'policy': {'2-of': [{'signed-by': 0}, {'signed-by': 1}]}}

        self.outof_complex = \
            "OutOf(2, 'Org1.member', 'Org2.member', 'Org3.member')"
        self.outof_complex_d = {'identities': [
            {'role': {'mspId': 'Org1', 'name': 'member'}},
            {'role': {'mspId': 'Org2', 'name': 'member'}},
            {'role': {'mspId': 'Org3', 'name': 'member'}}
        ],
            'policy': {'2-of': [
                {'signed-by': 0}, {'signed-by': 1}, {'signed-by': 2}
            ]}}
        # is equivalent to
        self.complex = "OR(AND('Org1.member', 'Org2.member')," \
                       " AND('Org1.member', 'Org3.member')," \
                       " AND('Org2.member', 'Org3.member'))"
        self.complex_d = {'identities': [
            {'role': {'mspId': 'Org1', 'name': 'member'}},
            {'role': {'mspId': 'Org2', 'name': 'member'}},
            {'role': {'mspId': 'Org3', 'name': 'member'}}
        ],
            'policy': {'1-of': [
                {'2-of': [{'signed-by': 0}, {'signed-by': 1}]},
                {'2-of': [{'signed-by': 0}, {'signed-by': 2}]},
                {'2-of': [{'signed-by': 1}, {'signed-by': 2}]}
            ]}}

        self._1ofAny = "OR('Org1.member', 'Org2.member'," \
                       " 'Org1.admin', 'Org2.admin')"
        self._1AdminOr2Other = "OR(AND('Org1.member', 'Org2.member')," \
                               " 'Org1.admin', 'Org2.admin')"
        self._2ofAny = "OutOf(2, 'Org1.member', 'Org2.member'," \
                       " 'Org1.admin', 'Org2.admin')"

        self.dumb_policy = {'identities': [
            {'role': {'name': 'member', 'mspId': 'chu-nantesMSP'}}
        ],
            'policy': {'signed-by': 0}}

    def test_s2d_outof_or(self):
        self.assertEqual(self.or_d, s2d().parse(self.outof_or))

    def test_s2d_or(self):
        self.assertEqual(self.or_d, s2d().parse(self.or_))

    def test_s2d_outof_and(self):
        self.assertEqual(self.and_d, s2d().parse(self.outof_and))

    def test_s2d_and_(self):
        self.assertEqual(self.and_d, s2d().parse(self.and_))

    def test_s2d_outof_complex(self):
        self.assertEqual(self.outof_complex_d, s2d().parse(
            self.outof_complex))

    def test_s2d_complex(self):
        self.assertEqual(self.complex_d, s2d().parse(self.complex))

    def test_s2d_1ofAny(self):
        self.assertEqual({'identities': [
            {'role': {'mspId': 'Org1', 'name': 'member'}},
            {'role': {'mspId': 'Org2', 'name': 'member'}},
            {'role': {'mspId': 'Org1', 'name': 'admin'}},
            {'role': {'mspId': 'Org2', 'name': 'admin'}}
        ],
            'policy': {'1-of': [
                {'signed-by': 0},
                {'signed-by': 1},
                {'signed-by': 2},
                {'signed-by': 3}
            ]}},
            s2d().parse(self._1ofAny))

    def test_s2d_1AdminOr2Other(self):
        self.assertEqual({'identities': [
            {'role': {'mspId': 'Org1', 'name': 'member'}},
            {'role': {'mspId': 'Org2', 'name': 'member'}},
            {'role': {'mspId': 'Org1', 'name': 'admin'}},
            {'role': {'mspId': 'Org2', 'name': 'admin'}}
        ],
            'policy': {
                '1-of': [
                    {'2-of': [{'signed-by': 0}, {'signed-by': 1}]},
                    {'signed-by': 2},
                    {'signed-by': 3}
                ]}},
            s2d().parse(self._1AdminOr2Other))

    def test_s2d_2ofAny(self):
        self.assertEqual({'identities': [
            {'role': {'mspId': 'Org1', 'name': 'member'}},
            {'role': {'mspId': 'Org2', 'name': 'member'}},
            {'role': {'mspId': 'Org1', 'name': 'admin'}},
            {'role': {'mspId': 'Org2', 'name': 'admin'}}
        ],
            'policy': {'2-of': [{'signed-by': 0},
                                {'signed-by': 1},
                                {'signed-by': 2},
                                {'signed-by': 3}]}},
            s2d().parse(self._2ofAny))

    def test_d2s_dumb_policy(self):
        self.assertEqual("OutOf(1, 'chu-nantesMSP.member')",
                         d2s.parse(self.dumb_policy))

    def test_d2s_outof_or(self):
        self.assertEqual(self.outof_or, d2s.parse(self.or_d))

    def test_d2s_outof_and(self):
        self.assertEqual(self.outof_and, d2s.parse(self.and_d))

    def test_d2s_s2d_outof_or(self):
        self.assertEqual(self.outof_or, d2s.parse(
            s2d().parse(self.outof_or)))

    def test_d2s_s2d_or(self):
        self.assertEqual(self.outof_or, d2s.parse(
            s2d().parse(self.or_)))

    def test_d2s_s2d_outof_complex(self):
        self.assertEqual(self.outof_complex, d2s.parse(
            s2d().parse(self.outof_complex)))


if __name__ == '__main__':
    unittest.main()



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest


class InstantiationTest(unittest.TestCase):
    """ Chaincode related Test cases
    """

    # @unittest.expectedFailure
    def test_create_instantiation(self):
        # TODO impl
        # trans_act_pro_hand = None
        # instantiation = Instantiation()
        # self.assertTrue(instantiation, Instantiation)
        pass


if __name__ == '__main__':
    unittest.main()



# SPDX-License-Identifier: Apache-2.0

import os
import subprocess
import time
import unittest
import uuid

from hfc.fabric.client import Client
from hfc.fabric.user import create_user
from test.integration.config import E2E_CONFIG


class BaseTestCase(unittest.TestCase):
    """
    Base class for test cases.
    All test cases can feel free to implement this.
    """

    def setUp(self):
        self.gopath_bak = os.environ.get('GOPATH', '')
        gopath = os.path.normpath(os.path.join(os.path.dirname(__file__),
                                               "../fixtures/chaincode"))
        os.environ['GOPATH'] = os.path.abspath(gopath)
        self.channel_tx = \
            E2E_CONFIG['test-network']['channel-artifacts']['channel.tx']
        self.compose_file_path = \
            E2E_CONFIG['test-network']['docker']['compose_file_tls']

        self.config_yaml = \
            E2E_CONFIG['test-network']['channel-artifacts']['config_yaml']
        self.channel_profile = \
            E2E_CONFIG['test-network']['channel-artifacts']['channel_profile']
        self.client = Client('test/fixtures/network.json')
        self.channel_name = "businesschannel"  # default application channel
        self.user = self.client.get_user('org1.example.com', 'Admin')
        self.assertIsNotNone(self.user, 'org1 admin should not be None')

        # Boot up the testing network
        self.shutdown_test_env()
        self.start_test_env()
        time.sleep(1)

    def tearDown(self):
        time.sleep(1)
        self.shutdown_test_env()

    def check_logs(self):
        cli_call(["docker-compose", "-f", self.compose_file_path, "logs",
                  "--tail=200"])

    def start_test_env(self):
        cli_call(["docker-compose", "-f", self.compose_file_path, "up", "-d"])

    def shutdown_test_env(self):
        cli_call(["docker-compose", "-f", self.compose_file_path, "down"])


class ChannelEventHubTestCase(BaseTestCase):

    evts = {}

    def onTxEvent(self, tx_id, tx_status, block_number):
        if tx_id in self.evts:
            if 'txEvents' not in self.evts[tx_id]:
                self.evts[tx_id]['txEvents'] = []
            self.evts[tx_id]['txEvents'] += [{
                'tx_status': tx_status,
                'block_number': block_number,
            }]

    def create_onCcEvent(self, _uuid):
        class CCEvent(object):
            def __init__(self, _uuid, evts, evt_tx_id):
                self.uuid = _uuid
                self.evts = evts  # keep reference, no copy
                self.evt_tx_id = evt_tx_id

            def cc_event(self, cc_event, block_number, tx_id, tx_status):
                if tx_id in self.evts:
                    if 'txEvents' not in self.evts[tx_id]:
                        self.evts[tx_id]['txEvents'] = []
                    self.evts[tx_id]['txEvents'] += [{
                        'cc_event': cc_event,
                        'tx_status': tx_status,
                        'block_number': block_number,
                    }]

                # unregister chaincode event if same tx_id
                # and disconnect as chaincode evt are unregister False
                if tx_id == self.evt_tx_id:
                    for x in self.evts[tx_id]['peer']:
                        if x['uuid'] == self.uuid:
                            x['channel_event_hub'].\
                                unregisterChaincodeEvent(x['cr'])
                            x['channel_event_hub'].disconnect()

        o = CCEvent(_uuid, self.evts, self.evt_tx_id)
        return o.cc_event

    def registerChaincodeEvent(self, tx_id, cc_name, cc_pattern,
                               channel_event_hub):
        _uuid = uuid.uuid4().hex
        self.evt_tx_id = tx_id
        cr = channel_event_hub.registerChaincodeEvent(
            cc_name, cc_pattern, onEvent=self.create_onCcEvent(_uuid))

        if self.evt_tx_id not in self.evts:
            self.evts[self.evt_tx_id] = {'peer': []}

        self.evts[self.evt_tx_id]['peer'] += [
            {
                'uuid': _uuid,
                'channel_event_hub': channel_event_hub,
                'cr': cr
            }
        ]


# This should be deprecated, and use client.get_user() API instead
def get_peer_org_user(org, user, state_store):
    """Loads the requested user for a given peer org
        and returns a user object.
    """

    peer_user_base_path = os.path.join(
        os.getcwd(),
        'test/fixtures/e2e_cli/crypto-config/peerOrganizations/{0}'
        '/users/{1}@{0}/msp/'.format(org, user)
    )

    key_path = os.path.join(
        peer_user_base_path, 'keystore/',
        E2E_CONFIG['test-network'][org]['users'][user]['private_key']
    )

    cert_path = os.path.join(
        peer_user_base_path, 'signcerts/',
        E2E_CONFIG['test-network'][org]['users'][user]['cert']
    )

    msp_id = E2E_CONFIG['test-network'][org]['mspid']

    return create_user(user, org, state_store, msp_id, key_path, cert_path)


def get_orderer_org_user(org='example.com', user='Admin', state_store=None):
    """Loads the admin user for a given orderer org and
        returns an user object.
        Currently, orderer org only has Admin

    """
    msp_path = os.path.join(
        os.getcwd(),
        'test/fixtures/e2e_cli/crypto-config/ordererOrganizations/'
        'example.com/users/Admin@example.com/msp/')

    key_path = os.path.join(
        msp_path, 'keystore/',
        E2E_CONFIG['test-network']['orderer']['users'][user]['private_key']
    )

    cert_path = os.path.join(
        msp_path, 'signcerts',
        E2E_CONFIG['test-network']['orderer']['users'][user]['cert']
    )
    msp_id = E2E_CONFIG['test-network']['orderer']['mspid']

    return create_user(user, org, state_store, msp_id, key_path, cert_path)


def cli_call(arg_list, expect_success=True, env=os.environ.copy()):
    """Executes a CLI command in a subprocess and return the results.

    Args:
        arg_list: a list command arguments
        expect_success: use False to return even if an error occurred
                        when executing the command
        env:

    Returns: (string, string, int) output message, error message, return code

    """
    p = subprocess.Popen(arg_list, stdout=subprocess.PIPE,
                         stderr=subprocess.PIPE, env=env)
    output, error = p.communicate()
    if p.returncode != 0:
        if output:
            print("Output:\n" + str(output))
        if error:
            print("Error Message:\n" + str(error))
        if expect_success:
            raise subprocess.CalledProcessError(
                p.returncode, arg_list, output)
    return output, error, p.returncode



#
# SPDX-License-Identifier: Apache-2.0

from hfc.protos.orderer import ab_pb2
from hfc.protos.common import common_pb2
from hfc.protos.peer import chaincode_pb2, transaction_pb2


def create_seek_info(start=None, stop=None, behavior="BLOCK_UNTIL_READY"):

    # build start
    if start is not None:
        seek_specified_start = ab_pb2.SeekSpecified()
        seek_specified_start.number = start
        seek_start = ab_pb2.SeekPosition()
        seek_start.specified.CopyFrom(seek_specified_start)
    else:
        seek_start = ab_pb2.SeekPosition()
        seek_start.newest.CopyFrom(ab_pb2.SeekNewest())

    # build stop
    if stop is not None:
        seek_specified_stop = ab_pb2.SeekSpecified()
        seek_specified_stop.number = stop
        seek_stop = ab_pb2.SeekPosition()
        seek_stop.specified.CopyFrom(seek_specified_stop)
    else:
        seek_stop = ab_pb2.SeekPosition()
        seek_stop.newest.CopyFrom(ab_pb2.SeekNewest())

    # seek info with all parts
    seek_info = ab_pb2.SeekInfo()
    seek_info.start.CopyFrom(seek_start)
    seek_info.stop.CopyFrom(seek_stop)

    seek_info.behavior = ab_pb2.SeekInfo.SeekBehavior.Value(behavior)

    return seek_info


def create_seek_payload(seek_header, seek_info):

    seek_payload = common_pb2.Payload()
    seek_payload.header.CopyFrom(seek_header)
    seek_payload.data = seek_info.SerializeToString()
    seek_payload_bytes = seek_payload.SerializeToString()

    return seek_payload_bytes


def create_cc_spec(chaincode_input, chaincode_id, type):

    chaincode_spec = chaincode_pb2.ChaincodeSpec()
    chaincode_spec.type = chaincode_pb2.ChaincodeSpec.Type.Value(type)
    chaincode_spec.chaincode_id.CopyFrom(chaincode_id)
    chaincode_spec.input.CopyFrom(chaincode_input)

    return chaincode_spec


def create_tx_payload(endorsements, tran_req):

    cc_action_payload = transaction_pb2.ChaincodeActionPayload()
    response = tran_req.responses[0]
    cc_action_payload.action.proposal_response_payload = \
        response.payload
    cc_action_payload.action.endorsements.extend(endorsements)
    cc_action_payload.chaincode_proposal_payload = tran_req.proposal.payload

    tran = transaction_pb2.Transaction()
    cc_tran_action = tran.actions.add()
    cc_tran_action.header = tran_req.header.signature_header
    cc_tran_action.payload = cc_action_payload.SerializeToString()
    tran_payload = common_pb2.Payload()
    tran_payload.header.channel_header = tran_req.header.channel_header
    tran_payload.header.signature_header = tran_req.header.signature_header
    tran_payload.data = tran.SerializeToString()

    tran_payload_bytes = tran_payload.SerializeToString()
    return tran_payload_bytes


def create_envelope(seek_payload_sig, seek_payload_bytes):

    envelope = common_pb2.Envelope()
    envelope.signature = seek_payload_sig
    envelope.payload = seek_payload_bytes

    return envelope



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest


class InvocationTest(unittest.TestCase):
    """ Chaincode related Test cases
    """
    # @unittest.expectedFailure

    def test_create_invocation(self):
        # TODO impl
        # trans_act_prop
        # invocation = Invocation()
        # self.assertTrue(invocation, Invocation)
        pass

    # @unittest.expectedFailure
    def test_chaincoder_handle(self):
        # TODO impl
        pass


if __name__ == '__main__':
    unittest.main()



# Copyright 281165273@qq.com. All Rights Reserved.
#
# SPDX-License-Identifier: Apache-2.0

from hfc.util.utils import create_serialized_identity


class TXContext(object):
    """A class represent Transaction context."""

    def __init__(self, user, crypto, tx_prop_req, prop_wait_time=-1):
        """ Construct transaction context

        :param prop_wait_time: proposal request wait timeout
        :param tx_prop_req: transaction proposal request
        :param user: user
        :param crypto: crypto
        :return: The created TXContext instance or None

        """
        self._tx_prop_req = tx_prop_req
        self._user = user
        self._crypto = crypto
        self._identity = create_serialized_identity(user)
        self._nonce = crypto.generate_nonce(24)
        hash_func = crypto.hash
        self._tx_id = hash_func(self._nonce + self._identity).hexdigest()
        self._prop_wait_time = prop_wait_time

    def get_attrs(self):
        return ",".join("{}={}"
                        .format(k, getattr(self, k))
                        for k in self.__dict__.keys())

    def __str__(self):
        return "[{}:{}]".format(self.__class__.__name__, self.get_attrs())

    @property
    def tx_id(self):
        """Get transaction id."""
        return self._tx_id

    @property
    def epoch(self):
        """Get epoch."""
        return 0

    @property
    def nonce(self):
        """Get nonce"""
        return self._nonce

    @property
    def identity(self):
        """Get identity"""
        return self._identity

    def sign(self, plain_text):
        """Sign the text

        :param plain_text: plain text
        :return: the signed text

        """
        return self._crypto.sign(self._user.enrollment.private_key,
                                 plain_text)

    @property
    def prop_wait_time(self):
        """Get proposal wait time"""
        return self._prop_wait_time

    @prop_wait_time.setter
    def prop_wait_time(self, proposal_wait_time):
        """Set proposal wait time

        :param proposal_wait_time: proposal wait time
        :return:

        """
        self._prop_wait_time = proposal_wait_time

    @property
    def tx_prop_req(self):
        """Get transaction proposal request"""
        return self._tx_prop_req

    @tx_prop_req.setter
    def tx_prop_req(self, tx_prop_req):
        """Set transaction proposal request

        :param tx_prop_req: transaction proposal request
        :return:

        """
        self._tx_prop_req = tx_prop_req

    @property
    def user(self):
        """Get request user"""
        return self._user

    @property
    def crypto(self):
        """Get"""
        return self._crypto


def validate(tx_context):
    """Validate transaction context

    :param tx_context: transaction context
    :return: transaction context if no error
    :raises ValueError: Invalid transaction context

    """
    if not tx_context:
        raise ValueError("Missing transaction context object")

    if not tx_context.crypto:
        raise ValueError("Missing 'crypto' parameter "
                         "in the transaction context object")

    if not tx_context.user:
        raise ValueError("Missing 'user' parameter "
                         "in the transaction context object")

    if not tx_context.tx_prop_req:
        raise ValueError("Missing 'tx_prop_req' parameter "
                         "in the transaction context object")
    return tx_context


def create_tx_context(user, crypto, tx_prop_req, prop_wait_time=-1):
    """Create transaction context

    :param tx_prop_req: transaction proposal request
    :param user: user
    :param crypto: crypto
    :param prop_wait_time: proposal wait time
    :return: a transaction context instance (Default value = -1)

    """
    tx_context = TXContext(user, crypto, tx_prop_req, prop_wait_time)
    return validate(tx_context)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hfc/protos/common/configtx.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from hfc.protos.common import common_pb2 as hfc_dot_protos_dot_common_dot_common__pb2
from hfc.protos.common import policies_pb2 as hfc_dot_protos_dot_common_dot_policies__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='hfc/protos/common/configtx.proto',
  package='common',
  syntax='proto3',
  serialized_options=_b('\n$org.hyperledger.fabric.protos.commonZ+github.com/hyperledger/fabric/protos/common'),
  serialized_pb=_b('\n hfc/protos/common/configtx.proto\x12\x06\x63ommon\x1a\x1ehfc/protos/common/common.proto\x1a hfc/protos/common/policies.proto\"W\n\x0e\x43onfigEnvelope\x12\x1e\n\x06\x63onfig\x18\x01 \x01(\x0b\x32\x0e.common.Config\x12%\n\x0blast_update\x18\x02 \x01(\x0b\x32\x10.common.Envelope\"\x9d\x03\n\x11\x43onfigGroupSchema\x12\x35\n\x06groups\x18\x01 \x03(\x0b\x32%.common.ConfigGroupSchema.GroupsEntry\x12\x35\n\x06values\x18\x02 \x03(\x0b\x32%.common.ConfigGroupSchema.ValuesEntry\x12\x39\n\x08policies\x18\x03 \x03(\x0b\x32\'.common.ConfigGroupSchema.PoliciesEntry\x1aH\n\x0bGroupsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12(\n\x05value\x18\x02 \x01(\x0b\x32\x19.common.ConfigGroupSchema:\x02\x38\x01\x1aH\n\x0bValuesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12(\n\x05value\x18\x02 \x01(\x0b\x32\x19.common.ConfigValueSchema:\x02\x38\x01\x1aK\n\rPoliciesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12)\n\x05value\x18\x02 \x01(\x0b\x32\x1a.common.ConfigPolicySchema:\x02\x38\x01\"\x13\n\x11\x43onfigValueSchema\"\x14\n\x12\x43onfigPolicySchema\"R\n\x06\x43onfig\x12\x10\n\x08sequence\x18\x01 \x01(\x04\x12*\n\rchannel_group\x18\x02 \x01(\x0b\x32\x13.common.ConfigGroupJ\x04\x08\x03\x10\x04R\x04type\"Z\n\x14\x43onfigUpdateEnvelope\x12\x15\n\rconfig_update\x18\x01 \x01(\x0c\x12+\n\nsignatures\x18\x02 \x03(\x0b\x32\x17.common.ConfigSignature\"\xf1\x01\n\x0c\x43onfigUpdate\x12\x12\n\nchannel_id\x18\x01 \x01(\t\x12%\n\x08read_set\x18\x02 \x01(\x0b\x32\x13.common.ConfigGroup\x12&\n\twrite_set\x18\x03 \x01(\x0b\x32\x13.common.ConfigGroup\x12=\n\risolated_data\x18\x05 \x03(\x0b\x32&.common.ConfigUpdate.IsolatedDataEntry\x1a\x33\n\x11IsolatedDataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x0c:\x02\x38\x01J\x04\x08\x04\x10\x05R\x04type\"\x98\x03\n\x0b\x43onfigGroup\x12\x0f\n\x07version\x18\x01 \x01(\x04\x12/\n\x06groups\x18\x02 \x03(\x0b\x32\x1f.common.ConfigGroup.GroupsEntry\x12/\n\x06values\x18\x03 \x03(\x0b\x32\x1f.common.ConfigGroup.ValuesEntry\x12\x33\n\x08policies\x18\x04 \x03(\x0b\x32!.common.ConfigGroup.PoliciesEntry\x12\x12\n\nmod_policy\x18\x05 \x01(\t\x1a\x42\n\x0bGroupsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\"\n\x05value\x18\x02 \x01(\x0b\x32\x13.common.ConfigGroup:\x02\x38\x01\x1a\x42\n\x0bValuesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\"\n\x05value\x18\x02 \x01(\x0b\x32\x13.common.ConfigValue:\x02\x38\x01\x1a\x45\n\rPoliciesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12#\n\x05value\x18\x02 \x01(\x0b\x32\x14.common.ConfigPolicy:\x02\x38\x01\"A\n\x0b\x43onfigValue\x12\x0f\n\x07version\x18\x01 \x01(\x04\x12\r\n\x05value\x18\x02 \x01(\x0c\x12\x12\n\nmod_policy\x18\x03 \x01(\t\"S\n\x0c\x43onfigPolicy\x12\x0f\n\x07version\x18\x01 \x01(\x04\x12\x1e\n\x06policy\x18\x02 \x01(\x0b\x32\x0e.common.Policy\x12\x12\n\nmod_policy\x18\x03 \x01(\t\">\n\x0f\x43onfigSignature\x12\x18\n\x10signature_header\x18\x01 \x01(\x0c\x12\x11\n\tsignature\x18\x02 \x01(\x0c\x42S\n$org.hyperledger.fabric.protos.commonZ+github.com/hyperledger/fabric/protos/commonb\x06proto3')
  ,
  dependencies=[hfc_dot_protos_dot_common_dot_common__pb2.DESCRIPTOR,hfc_dot_protos_dot_common_dot_policies__pb2.DESCRIPTOR,])




_CONFIGENVELOPE = _descriptor.Descriptor(
  name='ConfigEnvelope',
  full_name='common.ConfigEnvelope',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='config', full_name='common.ConfigEnvelope.config', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='last_update', full_name='common.ConfigEnvelope.last_update', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=110,
  serialized_end=197,
)


_CONFIGGROUPSCHEMA_GROUPSENTRY = _descriptor.Descriptor(
  name='GroupsEntry',
  full_name='common.ConfigGroupSchema.GroupsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='common.ConfigGroupSchema.GroupsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='common.ConfigGroupSchema.GroupsEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=390,
  serialized_end=462,
)

_CONFIGGROUPSCHEMA_VALUESENTRY = _descriptor.Descriptor(
  name='ValuesEntry',
  full_name='common.ConfigGroupSchema.ValuesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='common.ConfigGroupSchema.ValuesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='common.ConfigGroupSchema.ValuesEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=464,
  serialized_end=536,
)

_CONFIGGROUPSCHEMA_POLICIESENTRY = _descriptor.Descriptor(
  name='PoliciesEntry',
  full_name='common.ConfigGroupSchema.PoliciesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='common.ConfigGroupSchema.PoliciesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='common.ConfigGroupSchema.PoliciesEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=538,
  serialized_end=613,
)

_CONFIGGROUPSCHEMA = _descriptor.Descriptor(
  name='ConfigGroupSchema',
  full_name='common.ConfigGroupSchema',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='groups', full_name='common.ConfigGroupSchema.groups', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='values', full_name='common.ConfigGroupSchema.values', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='policies', full_name='common.ConfigGroupSchema.policies', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_CONFIGGROUPSCHEMA_GROUPSENTRY, _CONFIGGROUPSCHEMA_VALUESENTRY, _CONFIGGROUPSCHEMA_POLICIESENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=200,
  serialized_end=613,
)


_CONFIGVALUESCHEMA = _descriptor.Descriptor(
  name='ConfigValueSchema',
  full_name='common.ConfigValueSchema',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=615,
  serialized_end=634,
)


_CONFIGPOLICYSCHEMA = _descriptor.Descriptor(
  name='ConfigPolicySchema',
  full_name='common.ConfigPolicySchema',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=636,
  serialized_end=656,
)


_CONFIG = _descriptor.Descriptor(
  name='Config',
  full_name='common.Config',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sequence', full_name='common.Config.sequence', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='channel_group', full_name='common.Config.channel_group', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=658,
  serialized_end=740,
)


_CONFIGUPDATEENVELOPE = _descriptor.Descriptor(
  name='ConfigUpdateEnvelope',
  full_name='common.ConfigUpdateEnvelope',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='config_update', full_name='common.ConfigUpdateEnvelope.config_update', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='signatures', full_name='common.ConfigUpdateEnvelope.signatures', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=742,
  serialized_end=832,
)


_CONFIGUPDATE_ISOLATEDDATAENTRY = _descriptor.Descriptor(
  name='IsolatedDataEntry',
  full_name='common.ConfigUpdate.IsolatedDataEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='common.ConfigUpdate.IsolatedDataEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='common.ConfigUpdate.IsolatedDataEntry.value', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1013,
  serialized_end=1064,
)

_CONFIGUPDATE = _descriptor.Descriptor(
  name='ConfigUpdate',
  full_name='common.ConfigUpdate',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='channel_id', full_name='common.ConfigUpdate.channel_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='read_set', full_name='common.ConfigUpdate.read_set', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='write_set', full_name='common.ConfigUpdate.write_set', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='isolated_data', full_name='common.ConfigUpdate.isolated_data', index=3,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_CONFIGUPDATE_ISOLATEDDATAENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=835,
  serialized_end=1076,
)


_CONFIGGROUP_GROUPSENTRY = _descriptor.Descriptor(
  name='GroupsEntry',
  full_name='common.ConfigGroup.GroupsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='common.ConfigGroup.GroupsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='common.ConfigGroup.GroupsEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1282,
  serialized_end=1348,
)

_CONFIGGROUP_VALUESENTRY = _descriptor.Descriptor(
  name='ValuesEntry',
  full_name='common.ConfigGroup.ValuesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='common.ConfigGroup.ValuesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='common.ConfigGroup.ValuesEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1350,
  serialized_end=1416,
)

_CONFIGGROUP_POLICIESENTRY = _descriptor.Descriptor(
  name='PoliciesEntry',
  full_name='common.ConfigGroup.PoliciesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='common.ConfigGroup.PoliciesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='common.ConfigGroup.PoliciesEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1418,
  serialized_end=1487,
)

_CONFIGGROUP = _descriptor.Descriptor(
  name='ConfigGroup',
  full_name='common.ConfigGroup',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='version', full_name='common.ConfigGroup.version', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='groups', full_name='common.ConfigGroup.groups', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='values', full_name='common.ConfigGroup.values', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='policies', full_name='common.ConfigGroup.policies', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='mod_policy', full_name='common.ConfigGroup.mod_policy', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_CONFIGGROUP_GROUPSENTRY, _CONFIGGROUP_VALUESENTRY, _CONFIGGROUP_POLICIESENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1079,
  serialized_end=1487,
)


_CONFIGVALUE = _descriptor.Descriptor(
  name='ConfigValue',
  full_name='common.ConfigValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='version', full_name='common.ConfigValue.version', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='common.ConfigValue.value', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='mod_policy', full_name='common.ConfigValue.mod_policy', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1489,
  serialized_end=1554,
)


_CONFIGPOLICY = _descriptor.Descriptor(
  name='ConfigPolicy',
  full_name='common.ConfigPolicy',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='version', full_name='common.ConfigPolicy.version', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='policy', full_name='common.ConfigPolicy.policy', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='mod_policy', full_name='common.ConfigPolicy.mod_policy', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1556,
  serialized_end=1639,
)


_CONFIGSIGNATURE = _descriptor.Descriptor(
  name='ConfigSignature',
  full_name='common.ConfigSignature',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='signature_header', full_name='common.ConfigSignature.signature_header', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='signature', full_name='common.ConfigSignature.signature', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1641,
  serialized_end=1703,
)

_CONFIGENVELOPE.fields_by_name['config'].message_type = _CONFIG
_CONFIGENVELOPE.fields_by_name['last_update'].message_type = hfc_dot_protos_dot_common_dot_common__pb2._ENVELOPE
_CONFIGGROUPSCHEMA_GROUPSENTRY.fields_by_name['value'].message_type = _CONFIGGROUPSCHEMA
_CONFIGGROUPSCHEMA_GROUPSENTRY.containing_type = _CONFIGGROUPSCHEMA
_CONFIGGROUPSCHEMA_VALUESENTRY.fields_by_name['value'].message_type = _CONFIGVALUESCHEMA
_CONFIGGROUPSCHEMA_VALUESENTRY.containing_type = _CONFIGGROUPSCHEMA
_CONFIGGROUPSCHEMA_POLICIESENTRY.fields_by_name['value'].message_type = _CONFIGPOLICYSCHEMA
_CONFIGGROUPSCHEMA_POLICIESENTRY.containing_type = _CONFIGGROUPSCHEMA
_CONFIGGROUPSCHEMA.fields_by_name['groups'].message_type = _CONFIGGROUPSCHEMA_GROUPSENTRY
_CONFIGGROUPSCHEMA.fields_by_name['values'].message_type = _CONFIGGROUPSCHEMA_VALUESENTRY
_CONFIGGROUPSCHEMA.fields_by_name['policies'].message_type = _CONFIGGROUPSCHEMA_POLICIESENTRY
_CONFIG.fields_by_name['channel_group'].message_type = _CONFIGGROUP
_CONFIGUPDATEENVELOPE.fields_by_name['signatures'].message_type = _CONFIGSIGNATURE
_CONFIGUPDATE_ISOLATEDDATAENTRY.containing_type = _CONFIGUPDATE
_CONFIGUPDATE.fields_by_name['read_set'].message_type = _CONFIGGROUP
_CONFIGUPDATE.fields_by_name['write_set'].message_type = _CONFIGGROUP
_CONFIGUPDATE.fields_by_name['isolated_data'].message_type = _CONFIGUPDATE_ISOLATEDDATAENTRY
_CONFIGGROUP_GROUPSENTRY.fields_by_name['value'].message_type = _CONFIGGROUP
_CONFIGGROUP_GROUPSENTRY.containing_type = _CONFIGGROUP
_CONFIGGROUP_VALUESENTRY.fields_by_name['value'].message_type = _CONFIGVALUE
_CONFIGGROUP_VALUESENTRY.containing_type = _CONFIGGROUP
_CONFIGGROUP_POLICIESENTRY.fields_by_name['value'].message_type = _CONFIGPOLICY
_CONFIGGROUP_POLICIESENTRY.containing_type = _CONFIGGROUP
_CONFIGGROUP.fields_by_name['groups'].message_type = _CONFIGGROUP_GROUPSENTRY
_CONFIGGROUP.fields_by_name['values'].message_type = _CONFIGGROUP_VALUESENTRY
_CONFIGGROUP.fields_by_name['policies'].message_type = _CONFIGGROUP_POLICIESENTRY
_CONFIGPOLICY.fields_by_name['policy'].message_type = hfc_dot_protos_dot_common_dot_policies__pb2._POLICY
DESCRIPTOR.message_types_by_name['ConfigEnvelope'] = _CONFIGENVELOPE
DESCRIPTOR.message_types_by_name['ConfigGroupSchema'] = _CONFIGGROUPSCHEMA
DESCRIPTOR.message_types_by_name['ConfigValueSchema'] = _CONFIGVALUESCHEMA
DESCRIPTOR.message_types_by_name['ConfigPolicySchema'] = _CONFIGPOLICYSCHEMA
DESCRIPTOR.message_types_by_name['Config'] = _CONFIG
DESCRIPTOR.message_types_by_name['ConfigUpdateEnvelope'] = _CONFIGUPDATEENVELOPE
DESCRIPTOR.message_types_by_name['ConfigUpdate'] = _CONFIGUPDATE
DESCRIPTOR.message_types_by_name['ConfigGroup'] = _CONFIGGROUP
DESCRIPTOR.message_types_by_name['ConfigValue'] = _CONFIGVALUE
DESCRIPTOR.message_types_by_name['ConfigPolicy'] = _CONFIGPOLICY
DESCRIPTOR.message_types_by_name['ConfigSignature'] = _CONFIGSIGNATURE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ConfigEnvelope = _reflection.GeneratedProtocolMessageType('ConfigEnvelope', (_message.Message,), dict(
  DESCRIPTOR = _CONFIGENVELOPE,
  __module__ = 'hfc.protos.common.configtx_pb2'
  # @@protoc_insertion_point(class_scope:common.ConfigEnvelope)
  ))
_sym_db.RegisterMessage(ConfigEnvelope)

ConfigGroupSchema = _reflection.GeneratedProtocolMessageType('ConfigGroupSchema', (_message.Message,), dict(

  GroupsEntry = _reflection.GeneratedProtocolMessageType('GroupsEntry', (_message.Message,), dict(
    DESCRIPTOR = _CONFIGGROUPSCHEMA_GROUPSENTRY,
    __module__ = 'hfc.protos.common.configtx_pb2'
    # @@protoc_insertion_point(class_scope:common.ConfigGroupSchema.GroupsEntry)
    ))
  ,

  ValuesEntry = _reflection.GeneratedProtocolMessageType('ValuesEntry', (_message.Message,), dict(
    DESCRIPTOR = _CONFIGGROUPSCHEMA_VALUESENTRY,
    __module__ = 'hfc.protos.common.configtx_pb2'
    # @@protoc_insertion_point(class_scope:common.ConfigGroupSchema.ValuesEntry)
    ))
  ,

  PoliciesEntry = _reflection.GeneratedProtocolMessageType('PoliciesEntry', (_message.Message,), dict(
    DESCRIPTOR = _CONFIGGROUPSCHEMA_POLICIESENTRY,
    __module__ = 'hfc.protos.common.configtx_pb2'
    # @@protoc_insertion_point(class_scope:common.ConfigGroupSchema.PoliciesEntry)
    ))
  ,
  DESCRIPTOR = _CONFIGGROUPSCHEMA,
  __module__ = 'hfc.protos.common.configtx_pb2'
  # @@protoc_insertion_point(class_scope:common.ConfigGroupSchema)
  ))
_sym_db.RegisterMessage(ConfigGroupSchema)
_sym_db.RegisterMessage(ConfigGroupSchema.GroupsEntry)
_sym_db.RegisterMessage(ConfigGroupSchema.ValuesEntry)
_sym_db.RegisterMessage(ConfigGroupSchema.PoliciesEntry)

ConfigValueSchema = _reflection.GeneratedProtocolMessageType('ConfigValueSchema', (_message.Message,), dict(
  DESCRIPTOR = _CONFIGVALUESCHEMA,
  __module__ = 'hfc.protos.common.configtx_pb2'
  # @@protoc_insertion_point(class_scope:common.ConfigValueSchema)
  ))
_sym_db.RegisterMessage(ConfigValueSchema)

ConfigPolicySchema = _reflection.GeneratedProtocolMessageType('ConfigPolicySchema', (_message.Message,), dict(
  DESCRIPTOR = _CONFIGPOLICYSCHEMA,
  __module__ = 'hfc.protos.common.configtx_pb2'
  # @@protoc_insertion_point(class_scope:common.ConfigPolicySchema)
  ))
_sym_db.RegisterMessage(ConfigPolicySchema)

Config = _reflection.GeneratedProtocolMessageType('Config', (_message.Message,), dict(
  DESCRIPTOR = _CONFIG,
  __module__ = 'hfc.protos.common.configtx_pb2'
  # @@protoc_insertion_point(class_scope:common.Config)
  ))
_sym_db.RegisterMessage(Config)

ConfigUpdateEnvelope = _reflection.GeneratedProtocolMessageType('ConfigUpdateEnvelope', (_message.Message,), dict(
  DESCRIPTOR = _CONFIGUPDATEENVELOPE,
  __module__ = 'hfc.protos.common.configtx_pb2'
  # @@protoc_insertion_point(class_scope:common.ConfigUpdateEnvelope)
  ))
_sym_db.RegisterMessage(ConfigUpdateEnvelope)

ConfigUpdate = _reflection.GeneratedProtocolMessageType('ConfigUpdate', (_message.Message,), dict(

  IsolatedDataEntry = _reflection.GeneratedProtocolMessageType('IsolatedDataEntry', (_message.Message,), dict(
    DESCRIPTOR = _CONFIGUPDATE_ISOLATEDDATAENTRY,
    __module__ = 'hfc.protos.common.configtx_pb2'
    # @@protoc_insertion_point(class_scope:common.ConfigUpdate.IsolatedDataEntry)
    ))
  ,
  DESCRIPTOR = _CONFIGUPDATE,
  __module__ = 'hfc.protos.common.configtx_pb2'
  # @@protoc_insertion_point(class_scope:common.ConfigUpdate)
  ))
_sym_db.RegisterMessage(ConfigUpdate)
_sym_db.RegisterMessage(ConfigUpdate.IsolatedDataEntry)

ConfigGroup = _reflection.GeneratedProtocolMessageType('ConfigGroup', (_message.Message,), dict(

  GroupsEntry = _reflection.GeneratedProtocolMessageType('GroupsEntry', (_message.Message,), dict(
    DESCRIPTOR = _CONFIGGROUP_GROUPSENTRY,
    __module__ = 'hfc.protos.common.configtx_pb2'
    # @@protoc_insertion_point(class_scope:common.ConfigGroup.GroupsEntry)
    ))
  ,

  ValuesEntry = _reflection.GeneratedProtocolMessageType('ValuesEntry', (_message.Message,), dict(
    DESCRIPTOR = _CONFIGGROUP_VALUESENTRY,
    __module__ = 'hfc.protos.common.configtx_pb2'
    # @@protoc_insertion_point(class_scope:common.ConfigGroup.ValuesEntry)
    ))
  ,

  PoliciesEntry = _reflection.GeneratedProtocolMessageType('PoliciesEntry', (_message.Message,), dict(
    DESCRIPTOR = _CONFIGGROUP_POLICIESENTRY,
    __module__ = 'hfc.protos.common.configtx_pb2'
    # @@protoc_insertion_point(class_scope:common.ConfigGroup.PoliciesEntry)
    ))
  ,
  DESCRIPTOR = _CONFIGGROUP,
  __module__ = 'hfc.protos.common.configtx_pb2'
  # @@protoc_insertion_point(class_scope:common.ConfigGroup)
  ))
_sym_db.RegisterMessage(ConfigGroup)
_sym_db.RegisterMessage(ConfigGroup.GroupsEntry)
_sym_db.RegisterMessage(ConfigGroup.ValuesEntry)
_sym_db.RegisterMessage(ConfigGroup.PoliciesEntry)

ConfigValue = _reflection.GeneratedProtocolMessageType('ConfigValue', (_message.Message,), dict(
  DESCRIPTOR = _CONFIGVALUE,
  __module__ = 'hfc.protos.common.configtx_pb2'
  # @@protoc_insertion_point(class_scope:common.ConfigValue)
  ))
_sym_db.RegisterMessage(ConfigValue)

ConfigPolicy = _reflection.GeneratedProtocolMessageType('ConfigPolicy', (_message.Message,), dict(
  DESCRIPTOR = _CONFIGPOLICY,
  __module__ = 'hfc.protos.common.configtx_pb2'
  # @@protoc_insertion_point(class_scope:common.ConfigPolicy)
  ))
_sym_db.RegisterMessage(ConfigPolicy)

ConfigSignature = _reflection.GeneratedProtocolMessageType('ConfigSignature', (_message.Message,), dict(
  DESCRIPTOR = _CONFIGSIGNATURE,
  __module__ = 'hfc.protos.common.configtx_pb2'
  # @@protoc_insertion_point(class_scope:common.ConfigSignature)
  ))
_sym_db.RegisterMessage(ConfigSignature)


DESCRIPTOR._options = None
_CONFIGGROUPSCHEMA_GROUPSENTRY._options = None
_CONFIGGROUPSCHEMA_VALUESENTRY._options = None
_CONFIGGROUPSCHEMA_POLICIESENTRY._options = None
_CONFIGUPDATE_ISOLATEDDATAENTRY._options = None
_CONFIGGROUP_GROUPSENTRY._options = None
_CONFIGGROUP_VALUESENTRY._options = None
_CONFIGGROUP_POLICIESENTRY._options = None
# @@protoc_insertion_point(module_scope)
